
`timescale 1 ns / 1 ps
module LVDS41TX_1CLK6DATA 
(input wire clk_s 
,input wire clkop
,input wire clkos     
,input wire lock_chk         
,input wire reset       
,input wire [23:0] dataout
,output wire clkout      
,output wire clkout_n    
,output wire sclk         
,output wire tx_ready     
,output wire [5:0] dout    
,output wire [5:0] dout_n  
);

wire opensync_0;
wire opensync_1;
wire opensync_cken;
wire opensync_2;
wire buf_clkout;
wire scuba_vhi;
wire d35;
wire d25;
wire d15;
wire d05;
wire d34;
wire d24;
wire d14;
wire d04;
wire d33;
wire d23;
wire d13;
wire d03;
wire d32;
wire d22;
wire d12;
wire d02;
wire d31;
wire d21;
wire d11;
wire d01;
wire d30;
wire d20;
wire d10;
wire d00;
wire eclkc;
wire sclk_t;
wire cdiv1;
wire scuba_vlo;
wire eclkd;
wire xstop;
wire xstart;
wire opensync_3;
wire buf_douto5;
wire buf_douto4;
wire buf_douto3;
wire buf_douto2;
wire buf_douto1;
wire buf_douto0;

assign scuba_vhi = 1'b1;
assign scuba_vlo = 1'b0;

defparam LUT4_1.INIT =  16'h0a78 ;
LUT4 
LUT4_1 
(.I3(opensync_0)
,.I2(opensync_3)
,.I1(lock_chk)
,.I0(1)
,.F(opensync_cken)
);

defparam LUT4_0.INIT =  16'hfffe ;
LUT4 
LUT4_0 
(.I3(opensync_0)
,.I2(opensync_1)
,.I1(0)
,.I0(0)
,.F(xstop)
);

reg [3:0] opensync;
always@(posedge clk_s or posedge reset)
begin
  if(reset)
    opensync <= 4'b0001;
  else if(opensync_cken)
    opensync <= {opensync[2:0], opensync[3]};
end

assign {opensync_3, opensync_2, opensync_1, opensync_0} = opensync;

TLVDS_OBUF Inst7_OB (clkout, clkout_n, buf_clkout);

OSER4 Inst6_ODDRX2E 
(.D0(1)
,.D1(0)
,.D2(1)
,.D3(0)
,.TX0(1'b0)
,.TX1(1'b0)
,.FCLK(eclkc)
,.PCLK(sclk_t)
,.RESET(reset)
,.Q0(buf_clkout)
,.Q1()
);

OSER4 
Inst5_ODDRX2E5 
(.D0(d05)
,.D1(d15)
,.D2(d25)
,.D3(d35)
,.FCLK(eclkd)
,.TX0(1'b0)
,.TX1(1'b0)
,.PCLK(sclk_t)
,.RESET(reset)
,.Q0(buf_douto5)
,.Q1()
);

OSER4 
Inst5_ODDRX2E4 
(.D0(d04)
,.D1(d14)
,.D2(d24)
,.D3(d34)
,.FCLK(eclkd)
,.TX0(1'b0)
,.TX1(1'b0)
,.PCLK(sclk_t)
,.RESET(reset)
,.Q0(buf_douto4)
,.Q1()
);

OSER4 Inst5_ODDRX2E3 
(.D0(d03)
,.D1(d13)
,.D2(d23)
,.D3(d33)
,.FCLK(eclkd)
,.TX0(1'b0)
,.TX1(1'b0)
,.PCLK(sclk_t)
,.RESET(reset)
,.Q0(buf_douto3)
,.Q1()
);

OSER4 Inst5_ODDRX2E2 
(.D0(d02)
,.D1(d12)
,.D2(d22)
,.D3(d32)
,.FCLK(eclkd)
,.TX0(1'b0)
,.TX1(1'b0)
,.PCLK(sclk_t)
,.RESET(reset)
,.Q0(buf_douto2)
,.Q1()
);

OSER4 Inst5_ODDRX2E1 
(.D0(d01)
,.D1(d11)
,.D2(d21)
,.D3(d31)
,.FCLK(eclkd)
,.TX0(1'b0)
,.TX1(1'b0)
,.PCLK(sclk_t)
,.RESET(reset)
,.Q0(buf_douto1)
,.Q1()
);

OSER4 Inst5_ODDRX2E0 
(.D0(d00)
,.D1(d10)
,.D2(d20)
,.D3(d30)
,.FCLK(eclkd)
,.TX0(1'b0)
,.TX1(1'b0)
,.PCLK(sclk_t)
,.RESET(reset)
,.Q0(buf_douto0)
,.Q1()
);

assign eclkc = xstop ? 1'b0 : clkos;
assign eclkd = xstop ? 1'b0 : clkop;

defparam Inst3_CLKDIVC.DIV_MODE = "2" ;
CLKDIV Inst3_CLKDIVC 
(.RESETN(!reset)
,.HCLKIN(eclkd)
,.CALIB(scuba_vlo)
,.CLKOUT(sclk_t)
);

TLVDS_OBUF Inst1_OB5 (dout[5], dout_n[5], buf_douto5);
                               
TLVDS_OBUF Inst1_OB4 (dout[4], dout_n[4], buf_douto4);
                                   
TLVDS_OBUF Inst1_OB3 (dout[3], dout_n[3], buf_douto3);
                               
TLVDS_OBUF Inst1_OB2 (dout[2], dout_n[2], buf_douto2);
                                
TLVDS_OBUF Inst1_OB1 (dout[1], dout_n[1], buf_douto1);
                                  
TLVDS_OBUF Inst1_OB0 (dout[0], dout_n[0], buf_douto0);

assign sclk = sclk_t;
assign d35 = dataout[23];
assign d34 = dataout[22];
assign d33 = dataout[21];
assign d32 = dataout[20];
assign d31 = dataout[19];
assign d30 = dataout[18];
assign d25 = dataout[17];
assign d24 = dataout[16];
assign d23 = dataout[15];
assign d22 = dataout[14];
assign d21 = dataout[13];
assign d20 = dataout[12];
assign d15 = dataout[11];
assign d14 = dataout[10];
assign d13 = dataout[9];
assign d12 = dataout[8];
assign d11 = dataout[7];
assign d10 = dataout[6];
assign d05 = dataout[5];
assign d04 = dataout[4];
assign d03 = dataout[3];
assign d02 = dataout[2];
assign d01 = dataout[1];
assign d00 = dataout[0];
assign tx_ready = xstart;
assign xstart = opensync_3;

endmodule
