module bus_sync 
(clki
,datai
,clko
,datao
);
parameter     DATA_BITS = 8;
// inputs
input   wire                   clki;        // input clock
input   wire  [DATA_BITS-1:0]  datai;       // input data bus
input   wire                   clko;        // output clock
// outputs
output  reg   [DATA_BITS-1:0]  datao;       // output data bus

wire          xfer ;          // data stable to cross domains
reg     [DATA_BITS-1:0]  datax;       // captured data bus
reg     [1:0] counter = 2'b00;      // free-running div4, clki domain
reg           cnt1_d1;      // clko domain sync flop 
reg           cnt1_d2;      // clko domain sync flop 
reg           cnt1_d3;      // clko domain sync flop 

assign xfer = cnt1_d2 & !cnt1_d3;        

always @ (posedge clki) begin
    if(counter == 2'b01)
        datax <= datai;
end

always @ (posedge clki)  counter = counter + 1;
    
always @ (posedge clko) begin
    cnt1_d1 <= counter[1];
    cnt1_d2 <= cnt1_d1;
    cnt1_d3 <= cnt1_d2;
end

always @ (posedge clko) begin
    if(xfer)
        datao <= datax;
end

endmodule