`timescale 1ns/1ps
module lvds71_lvds41_top 
// Rx module ports 1CLK+8DATA
(input GSRN
,input RCLK_in
,input RA_in
,input RB_in
,input RC_in
,input RD_in
,input RE_in
,input RF_in
,input RG_in
,input RH_in
,input RCLK_in_n
,input RA_in_n
,input RB_in_n
,input RC_in_n
,input RD_in_n
,input RE_in_n
,input RF_in_n
,input RG_in_n
,input RH_in_n
// Tx module ports 4CLK+24DATA
,output TCLK0_out
,output [5:0] TDAT0_out
,output TCLK0_out_n
,output [5:0] TDAT0_out_n
,output TCLK1_out
,output [5:0] TDAT1_out
,output TCLK1_out_n
,output [5:0] TDAT1_out_n
,output TCLK2_out
,output [5:0] TDAT2_out
,output TCLK2_out_n
,output [5:0] TDAT2_out_n
,output TCLK3_out
,output [5:0] TDAT3_out
,output TCLK3_out_n
,output [5:0] TDAT3_out_n
// Status/Debug ports
,output [3:0] LED
);

wire [6:0] rx_a;
wire [6:0] rx_b;
wire [6:0] rx_c;
wire [6:0] rx_d;
wire [6:0] rx_e;
wire [6:0] rx_f;
wire [6:0] rx_g;
wire [6:0] rx_h;
wire [6:0] RCLK_out;
wire       rxpll_lock; 
wire       dphase_lock;
wire [3:0] dphase;   
wire       sclk;  
wire       eclk;
wire       stop;
wire       RST;
wire [7:0] r_R_0;  // Red,   8-bit data depth
wire [7:0] r_G_0;  // Green, 8-bit data depth
wire [7:0] r_B_0;  // Blue,  8-bit data depth
wire       r_Vsync_0;                        
wire       r_Hsync_0;                        
wire       r_DE_0   ;   
wire [7:0] r_R_1;  // Red,   8-bit data depth
wire [7:0] r_G_1;  // Green, 8-bit data depth
wire [7:0] r_B_1;  // Blue,  8-bit data depth
wire       r_Vsync_1;                        
wire       r_Hsync_1;                        
wire       r_DE_1   ; 
wire [23:0] tx_a;
wire [23:0] tx_b;
wire [23:0] tx_c;
wire [23:0] tx_d;
reg sclk_div2 = 0;
               
assign RST = ~ GSRN;

assign LED = {~dphase_lock, ~dphase[2:0]};

LVDS_7_to_1_RX rx_inst 
(.dphase_sel  (1'b0)
,.dphase_in   (4'b0)                  
,.RCLK_in     (RCLK_in)  // LVDS clock input pair
,.R0_in       (RA_in)    // LVDS data input pair 0
,.R1_in       (RB_in)    // LVDS data input pair 1
,.R2_in       (RC_in)    // LVDS data input pair 2
,.R3_in       (RD_in)    // LVDS data input pair 3   
,.R4_in       (RE_in)    // LVDS data input pair 4
,.R5_in       (RF_in)    // LVDS data input pair 5
,.R6_in       (RG_in)    // LVDS data input pair 6
,.R7_in       (RH_in)    // LVDS data input pair 7     
,.RCLK_in_n   (RCLK_in_n)// LVDS clock input pair
,.R0_in_n     (RA_in_n)  // LVDS data input pair 0
,.R1_in_n     (RB_in_n)  // LVDS data input pair 1
,.R2_in_n     (RC_in_n)  // LVDS data input pair 2
,.R3_in_n     (RD_in_n)  // LVDS data input pair 3   
,.R4_in_n     (RE_in_n)  // LVDS data input pair 4
,.R5_in_n     (RF_in_n)  // LVDS data input pair 5
,.R6_in_n     (RG_in_n)  // LVDS data input pair 6
,.R7_in_n     (RH_in_n)  // LVDS data input pair 7    
,.RCLK_out    (RCLK_out) // 7-bit parallel clock word
,.R0_out      (rx_a)     // 7-bit data output 0
,.R1_out      (rx_b)     // 7-bit data output 1
,.R2_out      (rx_c)     // 7-bit data output 2
,.R3_out      (rx_d)     // 7-bit data output 3
,.R4_out      (rx_e)     // 7-bit data output 4
,.R5_out      (rx_f)     // 7-bit data output 5
,.R6_out      (rx_g)     // 7-bit data output 6
,.R7_out      (rx_h)     // 7-bit data output 7
,.sclk        (sclk)     // parallel clock out
,.rxpll_lock  (rxpll_lock)              
,.eclk        (eclk)     // 3.5x clock from PLL
,.stop        (stop)     // eclksynca control signal out
,.dphase_lock (dphase_lock)
,.dphase_out  (dphase)
);
   
// Pin mapping from [rx_a, rx_b, rx_c, rx_d] to [r_R, r_G, r_B]
//                  bit-7     bit-6     bit-5     bit-4     bit-3     bit-2     bit-1     bit-0
assign  r_R_0 = { rx_a[5] , rx_a[6] , rx_d[1] , rx_d[2] , rx_d[3] , rx_d[4] , rx_d[5] , rx_d[6] };
assign  r_G_0 = { rx_a[3] , rx_a[4] , rx_c[2] , rx_c[3] , rx_c[4] , rx_c[5] , rx_c[6] , rx_d[0] };
assign  r_B_0 = { rx_a[1] , rx_a[2] , rx_b[3] , rx_b[4] , rx_b[5] , rx_b[6] , rx_c[0] , rx_c[1] };
// note: bit0 is received first in XO2 7:1 primitives
assign  r_Vsync_0 = rx_b[1];
assign  r_Hsync_0 = rx_b[2];
assign  r_DE_0    = rx_b[0];

// Pin mapping from [rx_a, rx_b, rx_c, rx_d] to [r_R, r_G, r_B]
//                  bit-7     bit-6     bit-5     bit-4     bit-3     bit-2     bit-1     bit-0
assign  r_R_1 = { rx_e[5] , rx_e[6] , rx_h[1] , rx_h[2] , rx_h[3] , rx_h[4] , rx_h[5] , rx_h[6] };
assign  r_G_1 = { rx_e[3] , rx_e[4] , rx_g[2] , rx_g[3] , rx_g[4] , rx_g[5] , rx_g[6] , rx_h[0] };
assign  r_B_1 = { rx_e[1] , rx_e[2] , rx_f[3] , rx_f[4] , rx_f[5] , rx_f[6] , rx_g[0] , rx_g[1] };
// note: bit0 is received first in GW2A 7:1 primitives
assign  r_Vsync_1 = rx_f[1];
assign  r_Hsync_1 = rx_f[2];
assign  r_DE_1    = rx_f[0];

//fifo 48bit->24bit
fifo48b_24b
u0_fifo48b_24b
(.rd_data  (tx_a)
,.rst_n    (!RST)
,.wr_data  ({r_R_1,r_G_1,r_B_1,r_R_0,r_G_0,r_B_0})
,.wr_en    (1)
,.wr_clk   (sclk)
,.rd_en    (1)
,.rd_clk   (clkop)
);

fifo48b_24b
u1_fifo48b_24b
(.rd_data  (tx_b)
,.rst_n    (!RST)
,.wr_data  ({r_R_1,r_G_1,r_B_1,r_R_0,r_G_0,r_B_0})
,.wr_en    (1)
,.wr_clk   (sclk)
,.rd_en    (1)
,.rd_clk   (clkop)
);

fifo48b_24b
u2_fifo48b_24b
(.rd_data  (tx_c)
,.rst_n    (!RST)
,.wr_data  ({r_R_1,r_G_1,r_B_1,r_R_0,r_G_0,r_B_0})
,.wr_en    (1)
,.wr_clk   (sclk)
,.rd_en    (1)
,.rd_clk   (clkop)
);

fifo48b_24b
u3_fifo48b_24b
(.rd_data  (tx_d)
,.rst_n    (!RST)
,.wr_data  ({r_R_1,r_G_1,r_B_1,r_R_0,r_G_0,r_B_0})
,.wr_en    (1)
,.wr_clk   (sclk)
,.rd_en    (1)
,.rd_clk   (clkop)
);

LVDS41TX_1CLK6DATA 
U_TX0
(.clk_s      (clkop)         
,.clkop      (clkop)        
,.clkos      (clkos)  
,.lock_chk   (lock_chk)   
,.reset      (RST)   
,.dataout    (tx_a)
,.clkout     (TCLK0_out)    
,.clkout_n   (TCLK0_out_n) 
,.sclk       (sclk_t0)   
,.tx_ready   (tx_ready0)    
,.dout       (TDAT0_out)    
,.dout_n     (TDAT0_out_n) 
);

LVDS41TX_1CLK6DATA 
U_TX1
(.clk_s      (clkop)         
,.clkop      (clkop)        
,.clkos      (clkos)  
,.lock_chk   (lock_chk)   
,.reset      (RST)   
,.dataout    (tx_b)
,.clkout     (TCLK1_out)    
,.clkout_n   (TCLK1_out_n)  
,.sclk       (sclk_t1)   
,.tx_ready   (tx_ready1)    
,.dout       (TDAT1_out)    
,.dout_n     (TDAT1_out_n)  
);

LVDS41TX_1CLK6DATA 
U_TX2
(.clk_s      (clkop)         
,.clkop      (clkop)        
,.clkos      (clkos)  
,.lock_chk   (lock_chk)          
,.reset      (RST)   
,.dataout    (tx_c)
,.clkout     (TCLK2_out)   
,.clkout_n   (TCLK2_out_n) 
,.sclk       (sclk_t2)   
,.tx_ready   (tx_ready2)    
,.dout       (TDAT2_out)    
,.dout_n     (TDAT2_out_n)   
);

LVDS41TX_1CLK6DATA 
U_TX3
(.clk_s      (clkop)         
,.clkop      (clkop)        
,.clkos      (clkos)  
,.lock_chk   (lock_chk)         
,.reset      (RST)   
,.dataout    (tx_d)
,.clkout     (TCLK3_out)   
,.clkout_n   (TCLK3_out_n)  
,.sclk       (sclk_t3)   
,.tx_ready   (tx_ready3)    
,.dout       (TDAT3_out)    
,.dout_n     (TDAT3_out_n)  
);

PLL 
pll_inst 
(.CLKOUT(clkop)
,.LOCK(lock_chk)
,.CLKOUTP(clkos)
,.CLKOUTD()
,.CLKOUTD3()
,.RESET(RST)
,.RESET_P(0)
,.RESET_I(0)
,.RESET_S(0)
,.CLKIN(sclk)
,.CLKFB(0)
,.FBDSEL(0)
,.IDSEL(0)
,.ODSEL(0)
,.PSDA(0)
,.DUTYDA(0)
,.FDLY(15)
);

defparam pll_inst.FCLKIN = "100";
defparam pll_inst.DYN_IDIV_SEL = "false";
defparam pll_inst.IDIV_SEL = 0;
defparam pll_inst.DYN_FBDIV_SEL = "false";
defparam pll_inst.FBDIV_SEL = 1;
defparam pll_inst.DYN_ODIV_SEL= "false";
defparam pll_inst.ODIV_SEL = 4;
defparam pll_inst.PSDA_SEL = "0100";
defparam pll_inst.DYN_DA_EN = "false";
defparam pll_inst.DUTYDA_SEL = "1000";
defparam pll_inst.CLKOUT_FT_DIR = 1'b1;
defparam pll_inst.CLKOUTP_FT_DIR = 1'b1;
defparam pll_inst.CLKOUT_DLY_STEP = 0;
defparam pll_inst.CLKOUTP_DLY_STEP = 0;
defparam pll_inst.CLKFB_SEL = "internal";
defparam pll_inst.CLKOUT_BYPASS = "false";
defparam pll_inst.CLKOUTP_BYPASS = "false";
defparam pll_inst.CLKOUTD_BYPASS = "false";
defparam pll_inst.DYN_SDIV_SEL = 2;
defparam pll_inst.CLKOUTD_SRC = "CLKOUT";
defparam pll_inst.CLKOUTD3_SRC = "CLKOUT";
defparam pll_inst.DEVICE = "GW2A-18";   
         
endmodule