`timescale 1 ns/ 1 ps
module LVDS_7_to_1_RX 
(input       dphase_sel  // 0: auto phase, 1: manual phase from DIP switches 
,input [3:0] dphase_in   // from DIP switches
,input       RCLK_in     // LVDS clock input pair
,input       R0_in       // LVDS data input pair 0
,input       R1_in       // LVDS data input pair 1
,input       R2_in       // LVDS data input pair 2
,input       R3_in       // LVDS data input pair 3  
,input       R4_in       // LVDS data input pair 4
,input       R5_in       // LVDS data input pair 5
,input       R6_in       // LVDS data input pair 6
,input       R7_in       // LVDS data input pair 7 
,input       RCLK_in_n   // LVDS clock input pair
,input       R0_in_n     // LVDS data input pair 0
,input       R1_in_n     // LVDS data input pair 1
,input       R2_in_n     // LVDS data input pair 2
,input       R3_in_n     // LVDS data input pair 3  
,input       R4_in_n     // LVDS data input pair 4
,input       R5_in_n     // LVDS data input pair 5
,input       R6_in_n     // LVDS data input pair 6
,input       R7_in_n     // LVDS data input pair 7 
,output [6:0] RCLK_out   // 7-bit parallel clock data
,output [6:0] R0_out     // 7-bit parallel data
,output [6:0] R1_out     // 7-bit parallel data
,output [6:0] R2_out     // 7-bit parallel data
,output [6:0] R3_out     // 7-bit parallel data       
,output [6:0] R4_out     // 7-bit parallel data
,output [6:0] R5_out     // 7-bit parallel data
,output [6:0] R6_out     // 7-bit parallel data
,output [6:0] R7_out     // 7-bit parallel data 
,output       eclk       // PLL output, to slave Tx if necessary
,output       stop       // eclksynca control signal out (to transmitter, if necessary)
,output       sclk       // System clock, Rcv outputs clocked on rising edge
,output       dphase_lock// status output, from bit_align module
,output [3:0] dphase_out // selected phase, from bit_align module
,output       rxpll_lock // Rx PLL lock outp
);

wire [3:0]  dphase; 	           // output state from bit_align module
reg         reset_sync;          // reset to all modules
wire [7:0]  rx_di;               // 8 serial input data streams
wire [7:0]  rx_di_n;             // 8 serial input data streams
wire        pll_phase_dir;
wire        pll_phase_step;
wire        slip;
wire        pll_lock;            // PLL locked status
wire  [6:0] rxdata_out [7:0];    // array of 4 7-bit recovered data words
wire  [6:0] RCLK_out_sclk;       // 7-bit parallel clock data, sclk domain
wire  [6:0] RCLK_out_rclk;       // 7-bit parallel clock data, RCLK_in domain      
wire        lost_sync;           // detected PLL loss-of-lock
reg  [18:0] delay_count = 0;     // delay after RCLK_in established
reg         release_reset = 0;   // asserts at end of delay
reg         pll_lock_1;          // sync pipe
reg         pll_lock_sync;       // sync pipe
reg         dphase_lock_d1;      // sync pipe
reg         dphase_lock_d2;      // sync pipe
wire        clk_diff;

assign dphase_out = !dphase_sel ? dphase : dphase_in;   // 0: auto phase, 1: manual phase from DIP switches 
assign rx_di = {R7_in, R6_in, R5_in, R4_in, R3_in, R2_in, R1_in, R0_in};
assign rx_di_n = {R7_in_n, R6_in_n, R5_in_n, R4_in_n, R3_in_n, R2_in_n, R1_in_n, R0_in_n};
assign rxpll_lock = pll_lock_sync;
assign RCLK_out = RCLK_out_sclk;
assign R0_out = rxdata_out[0];
assign R1_out = rxdata_out[1];
assign R2_out = rxdata_out[2];
assign R3_out = rxdata_out[3];
assign R4_out = rxdata_out[4];
assign R5_out = rxdata_out[5];
assign R6_out = rxdata_out[6];
assign R7_out = rxdata_out[7];
assign stop   = !pll_lock_sync;  // ensure transmitter sclk circuits have same phase

LVDS71RX_1CLK8DATA lvds_71_rx 
(.clk        (RCLK_in)
,.clk_n      (RCLK_in_n)
,.clk_s      (clk_diff)
,.init       (1'b1)
,.reset      (reset_sync)
,.pll_reset  (reset_sync)
,.datain     (rx_di)
,.datain_n   (rx_di_n)
,.phase_dir  (pll_phase_dir)
,.phase_step (pll_phase_step)
,.alignwd    (slip)
,.lock       (pll_lock)         //output
,.eclk       (eclk)             //output
,.sclk       (sclk)             //output
,.rx_ready   ()                 //output
,.clk_phase  (RCLK_out_sclk)
,.q0         (rxdata_out[0])
,.q1         (rxdata_out[1])
,.q2         (rxdata_out[2])
,.q3         (rxdata_out[3])
,.q4         (rxdata_out[4])
,.q5         (rxdata_out[5])
,.q6         (rxdata_out[6])
,.q7         (rxdata_out[7])
,.clk_diff   (clk_diff)
); 

bus_sync # (.DATA_BITS (7))
bus_sync_inst
(.clki      (sclk) 
,.datai     (RCLK_out_sclk)  // 7-bit clock word from Rcv module
,.clko      (clk_diff)  
,.datao     (RCLK_out_rclk)  // clock word to word_align module
);

bit_align_ctl bit_aln_ctl_inst
(.rx_clk      (clk_diff)
,.reset       (reset_sync)  
,.bit_aln_en  (pll_lock_sync)  // slave to PLL
,.rxclk_word  (RCLK_out_rclk)
,.phase_dir   (pll_phase_dir)
,.phase_rot   (pll_phase_step)
,.dphase      (dphase)
,.dphase_lock (dphase_lock)
);      

word_align_ctl wd_aln_ctl_inst
(.clk             (sclk) 
,.rst             (reset_sync)  
,.clock_word      (RCLK_out_sclk) 
,.align_enable    (dphase_lock_d2) // slave to bit_align_ctl and PLL
,.slip            (slip) 
);
        
always @(posedge sclk or posedge reset_sync) begin
    if(reset_sync == 1'b1) begin
        dphase_lock_d1 <= 0;
        dphase_lock_d2 <= 0;
    end else begin
        dphase_lock_d1 <= dphase_lock && pll_lock_sync;
        dphase_lock_d2 <= dphase_lock_d1;
    end
end

always @(posedge clk_diff or negedge pll_lock) begin
   if(pll_lock == 1'b0) begin
       pll_lock_1    <= 1'b0;
       pll_lock_sync <= 1'b0;
   end else begin
       pll_lock_1    <= 1'b1;
       pll_lock_sync <= pll_lock_1;
   end
end

assign lost_sync = 0;
   
always @(posedge clk_diff or posedge lost_sync) begin
    if(lost_sync == 1'b1) 
        delay_count <= 0;
    else if(delay_count[18] == 1'b0) 
        delay_count <= delay_count + 1;
end
    
always @(posedge clk_diff or posedge lost_sync) begin
    if(lost_sync == 1'b1) 
        release_reset <= 1'b0;
    else if(release_reset == 1'b0) 
        release_reset <= delay_count[17];
end 
 
always @(posedge clk_diff or posedge lost_sync) begin
   if(lost_sync == 1'b1) begin
       reset_sync <= 1'b1;
   end else begin
       reset_sync <= !release_reset;
   end
end

endmodule